@@ -66,7 +66,7 @@
 extern int errno;
 #endif
 
-#if defined(SOLARIS) || defined(__FreeBSD__)
+#if defined(SOLARIS) || defined(__FreeBSD__) || defined(__linux__)
 #  include <unistd.h>
 #  include <stdlib.h>
 #  include <string.h>
@@ -189,8 +189,15 @@ char *gethost(addr)
     hp = gethostbyaddr((char *) addr, sizeof(struct in_addr), AF_INET);
     if (hp)
     {
+	char *hname = strdup(hp->h_name);
+
+	if (! hname) {
+	    syslog(LOG_ERR, "strdup(%s): %m", hp->h_name);
+	    exit(1);
+	}
 	/* Found a IP -> Name match, now try the reverse for security reasons */
-	hp = gethostbyname(hp->h_name);
+	hp = gethostbyname(hname);
+	(void) free(hname);
 	if (hp)
 #ifdef h_addr
 	    for (i = 0; hp->h_addr_list[i]; i++)
